import os
import click
from merakitools.api.server import app

@click.group()
def cli():
	""" Start Meraki SYNC API Server """
	pass


@click.command(name='start',help="Starts API Server")
@click.option('--debug/--no-debug', help="Enables Debug Mode",default=False)
def start(debug):
	if debug:
		app.run(debug=True,host="0.0.0.0",port=5050)
	else:
		app.run(host="0.0.0.0",port=5050)


cli.add_command(start)


def meraki_create_sgt(db, organizationId, **kwargs):
    metadata = {
        'tags': ['organizations', 'configure', 'adaptivePolicy', 'groups'],
        'operation': 'createOrganizationAdaptivePolicyGroup',
    }
    resource = f'/organizations/{organizationId}/adaptivePolicy/groups'

    body_params = ['value', 'name', 'description', 'networkObjectId']
    payload = {k: v for (k, v) in kwargs.items() if k in body_params}

    return db._session.post(metadata, resource, payload)


def meraki_read_sgt(db, organizationId):
    metadata = {
        'tags': ['organizations', 'configure', 'adaptivePolicy', 'groups'],
        'operation': 'getOrganizationAdaptivePolicyGroups',
    }
    resource = f'/organizations/{organizationId}/adaptivePolicy/groups'

    return db._session.get(metadata, resource)


def meraki_update_sgt(db, organizationId, groupId: str, **kwargs):
    metadata = {
        'tags': ['organizations', 'configure', 'adaptivePolicy', 'groups'],
        'operation': 'updateOrganizationAdaptivePolicyGroup',
    }
    resource = f'/organizations/{organizationId}/adaptivePolicy/groups/{groupId}'

    if "value" in kwargs:
        post_resource = f'/organizations/{organizationId}/adaptivePolicy/groups'

        body_params = ['name', 'description', 'networkObjectId', 'value']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        db._session.delete(metadata, resource)
        new = db._session.post(metadata, post_resource, payload)
        return new
    else:
        body_params = ['name', 'description', 'networkObjectId']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return db._session.put(metadata, resource, payload)


def meraki_delete_sgt(db, organizationId, groupId: str):
    metadata = {
        'tags': ['organizations', 'configure', 'adaptivePolicy', 'groups'],
        'operation': 'deleteOrganizationAdaptivePolicyGroup',
    }
    resource = f'/organizations/{organizationId}/adaptivePolicy/groups/{groupId}'

    return db._session.delete(metadata, resource)


def meraki_create_sgacl(db, organizationId, **kwargs):
    if 'ipVersion' in kwargs:
        options = ['agnostic', 'ipv4', 'ipv6']
        assert kwargs[
                   'ipVersion'] in options, f'''"ipVersion" cannot be "{kwargs['ipVersion']}", & must be set to \
                   one of: {options}'''

    metadata = {
        'tags': ['organizations', 'configure', 'adaptivePolicy', 'acls'],
        'operation': 'createOrganizationAdaptivePolicyAcl',
    }
    resource = f'/organizations/{organizationId}/adaptivePolicy/acls'

    body_params = ['name', 'description', 'rules', 'ipVersion']
    payload = {k: v for (k, v) in kwargs.items() if k in body_params}

    return db._session.post(metadata, resource, payload)


def meraki_read_sgacl(db, organizationId):
    metadata = {
        'tags': ['organizations', 'configure', 'adaptivePolicy', 'acls'],
        'operation': 'getOrganizationAdaptivePolicyAcls',
    }
    resource = f'/organizations/{organizationId}/adaptivePolicy/acls'

    return db._session.get(metadata, resource)


def meraki_update_sgacl(db, organizationId, aclId: str, **kwargs):
    if 'ipVersion' in kwargs:
        options = ['agnostic', 'ipv4', 'ipv6']
        assert kwargs[
                   'ipVersion'] in options, f'''"ipVersion" cannot be "{kwargs['ipVersion']}", & must be set to \
                   one of: {options}'''

    metadata = {
        'tags': ['organizations', 'configure', 'adaptivePolicy', 'acls'],
        'operation': 'updateOrganizationAdaptivePolicyAcl',
    }
    resource = f'/organizations/{organizationId}/adaptivePolicy/acls/{aclId}'

    body_params = ['name', 'description', 'rules', 'ipVersion']
    payload = {k: v for (k, v) in kwargs.items() if k in body_params}

    return db._session.put(metadata, resource, payload)


def meraki_delete_sgacl(db, organizationId, aclId: str):
    metadata = {
        'tags': ['organizations', 'configure', 'adaptivePolicy', 'acls'],
        'operation': 'deleteOrganizationAdaptivePolicyAcl',
    }
    resource = f'/organizations/{organizationId}/adaptivePolicy/acls/{aclId}'

    return db._session.delete(metadata, resource)


def meraki_read_sgpolicy(db, organizationId):
    metadata = {
        'tags': ['organizations', 'configure', 'adaptivePolicy', 'bindings'],
        'operation': 'getOrganizationAdaptivePolicyBindings',
    }
    resource = f'/organizations/{organizationId}/adaptivePolicy/bindings'

    return db._session.get(metadata, resource)


def meraki_update_sgpolicy(db, organizationId, **kwargs):
    if 'catchAllRule' in kwargs:
        options = ['global', 'deny all', 'allow all']
        assert kwargs[
                   'catchAllRule'] in options, f'''"catchAllRule" cannot be "{kwargs['catchAllRule']}", & must be set \
                   to one of: {options}'''

    metadata = {
        'tags': ['organizations', 'configure', 'adaptivePolicy', 'bindings'],
        'operation': 'updateOrganizationAdaptivePolicyBindings',
    }
    resource = f'/organizations/{organizationId}/adaptivePolicy/bindings'

    body_params = ['srcGroupId', 'dstGroupId', 'name', 'description', 'aclIds', 'catchAllRule', 'bindingEnabled',
                   'monitorModeEnabled']
    payload = {k: v for (k, v) in kwargs.items() if k in body_params}

    return db._session.put(metadata, resource, payload)



import asyncio
import datetime
import requests
import random
import json
import lib
from merakitools import meraki_tasks
from merakitools import const, lib, model

import utils
from deepdiff import DeepHash
import pytz, dateutil.parser
import pprint
def setup_app(cfg_file=None):
    const.appcfg = model.APPCONFIG(cfg_file)
    temp_sdk = lib.MerakiApi()
    const.meraki_sdk = temp_sdk.api
    
def get_networks(sdk,org_id):
    return sdk.api.organizations.getOrganizationNetworks(org_id)
def uniqid():
    from time import time
    return hex(int(time()*10000000))[2:]
def get_netwok_events(sdk,networks):
    count = 0
    time_fmt = '%H:%M:%S'
    date_fmt = '%m/%d/%Y'
    full_fmt = '%Y-%m-%dT%H:%M:%S%z'
    date_start = "2021-09-21T00:00:00Z"
    date_stop = dateutil.parser.parse('2021-09-25T00:00:00Z')
    headers = {
            'Authorization': 'Basic am9zaDpDb3JlZnVja2VkbWUxIQ==',
            'Content-Type' : 'application/json'
    }
    url = f"https://search-merakilogs-pvyijv5ikz7ti5kd76kk56qxem.us-east-1.es.amazonaws.com/_bulk"
    for network in networks:

        events = sdk.api.networks.getNetworkEvents(networkId=network['id'],startingAfter=date_start,includedEventTypes=["wpa_auth","association"],perPage=1000)
        event_list = events['events']
        pageEnd = dateutil.parser.parse(events['pageEndAt'])
        pageEnd = pageEnd + datetime.timedelta(0,1)
        timeDiff = pageEnd > date_stop
        pageEnd = pageEnd.strftime(full_fmt)
        
        while not timeDiff:
            events = sdk.api.networks.getNetworkEvents(
                networkId=network['id'],
                startingAfter=pageEnd,
                includedEventTypes=["wpa_auth","association",], perPage=1000)
            print(f"Page Start Time: {events['pageStartAt']} Page End Time: {events['pageEndAt']}")
            event_list.extend(events['events'])
            pageEnd = dateutil.parser.parse(events['pageEndAt'])
            timeDiff = pageEnd > date_stop
            pageEnd = pageEnd + datetime.timedelta(0, 1)
            pageEnd = pageEnd.strftime(full_fmt)
        payload = ""
        for event in event_list:
            #print(event)
            utctime = dateutil.parser.parse(event['occurredAt'])
            localtime = utctime.astimezone(pytz.timezone('US/Eastern'))
            #print(f"{localtime.strftime(date_fmt)}-{localtime.strftime(time_fmt)}:{event['clientId']}-{event['clientDescription']} Connected to {network['name']} - {event['ssidName']} " )
            count = count+1
            #id = random.random()+count
            meta = json.dumps({ "create": { "_index": "meraki-wireless", "_id": uniqid()}})
            event.pop("occurredAt")
            event["timeStamp"] = localtime.strftime(full_fmt)
            data = json.dumps(event)
            payload=f"{payload}{meta}\n{data}\n"
	        
        if len(payload) > 0:
            response = requests.request("POST", url, headers=headers,
                                        data=payload)
        print(f'Total associations: {count}')
    print(f'Total associations: {count}')
def main(cfg_file):
    
    setup_app(cfg_file)
    sdk = lib.MerakiApi()
    
    all_orgs = sdk.api.organizations.getOrganizations()
    org_ids = []
    event_thread = []
    for name in const.appcfg.allow_org_list_names:
        for org in all_orgs:
            if DeepHash(str(name).upper()) == DeepHash(
                    str(org['name']).upper()):
                networks = get_networks(sdk, org['id'])
                event_thread.append(utils.NetworkEvents(org['id'],networks))
                org_ids.append(org['id'])
                
                #get_netwok_events(sdk, networks
    for thread in event_thread:
        thread.start()
    for thread in event_thread:
        thread.join()

    print(org_ids)
    
if __name__ == '__main__':
    config_file = '~/apps/testSync/config-doe.json'
    main(config_file)
    print('Done')



import asyncio
import datetime
import pytz, dateutil.parser
import requests
import json
import threading
from merakitools import lib

class NetworkEvents(threading.Thread):
	def __init__(self, org_id,networks):
		"""
		Init Function or Validateorginization
		Args:
			org_id(string): Orginization ID from Meraki
		"""
		threading.Thread.__init__(self)
		self.org_id = org_id
		self.networks = networks
	
	def run(self):
		"""
		Start the ASYNC fun fuctions and waits for completions to restore
		Cache
		Returns:

		"""
		asyncio.run(self._async_run())
	
	async def _async_run(self):
		"""
		Async version of the run function loops through all networks
		then sends the network to the valate proccessor
		Returns:

		"""
		print(
				f'\tOrgName: {self.org_id}Thread PID:{threading.currentThread().native_id}'
		)
		count = 0
		time_fmt = '%H:%M:%S'
		date_fmt = '%m/%d/%Y'
		full_fmt = '%Y-%m-%dT%H:%M:%S%z'
		date_start = "2021-09-29T00:00:00Z"
		date_stop = dateutil.parser.parse('2021-09-30T00:00:00Z')
		meta = json.dumps({"index": {}})
		headers = {
				'Authorization': 'Basic am9zaDpDb3JlZnVja2VkbWUxIQ==',
				'Content-Type' : 'application/json'
		}
		url = f"https://es.ciscolabnet.com/doe-meraki-wireless/_bulk"
		with lib.MerakiAsyncApi() as sdk:
			for network in self.networks:
				try:
					events = await sdk.networks.getNetworkEvents(
						networkId=network['id'], startingAfter=date_start,
						includedEventTypes=["wpa_auth", "association"],
						perPage=1000)
					event_list = events['events']
					pageEnd = dateutil.parser.parse(events['pageEndAt'])
					pageEnd = pageEnd + datetime.timedelta(0, 1)
					timeDiff = pageEnd > date_stop
					pageEnd = pageEnd.strftime(full_fmt)
				except Exception as e:
					print(f"Network {network['name']} - End time: {pageEnd} Error with get events {str(e)}")
					
				
				while not timeDiff:
					try:
						events = await sdk.networks.getNetworkEvents(
								networkId=network['id'],
								startingAfter=pageEnd,
								includedEventTypes=["wpa_auth",
								                    "association", ],
								perPage=1000)
						print(
							f"Network: {network['name']} - Page Start Time: {events['pageStartAt']} Page End Time: {events['pageEndAt']}")
						event_list.extend(events['events'])
						pageEnd = dateutil.parser.parse(events['pageEndAt'])
						timeDiff = pageEnd > date_stop
						pageEnd = pageEnd + datetime.timedelta(0, 1)
						pageEnd = pageEnd.strftime(full_fmt)
					except Exception as e:
						print(f"Network {network['name']} - End time: {pageEnd} Error with get events {str(e)}")
				payload = ""
				print(
					f'Building Payload for Network: {network["name"]} Event list size: {len(event_list)}')
				for event in event_list:
					# print(event)
					try:
						utctime = dateutil.parser.parse(event['occurredAt'])
						localtime = utctime.astimezone(
							pytz.timezone('US/Eastern'))
						# print(f"{localtime.strftime(date_fmt)}-{localtime.strftime(time_fmt)}:{event['clientId']}-{event['clientDescription']} Connected to {network['name']} - {event['ssidName']} " )
						count = count + 1
						# id = random.random()+count
						
						event.pop("occurredAt")
						event["timeStamp"] = localtime.strftime(full_fmt)
						event["networkName"] = network["name"]
						data = json.dumps(event)
						payload += f"{meta}\n{data}\n"
					except Exception as e:
						print(f"Error Building payload for Network: {network['name']} Error: {str(e)}")
				
				if len(payload) > 0:
					print(f'Network: {network["name"]} - Payload Size: {len(event_list)}')
					try:
						response = requests.request("POST", url,
						                            headers=headers,
						                            data=payload)
						print(f'Network: {network["name"]} fished eith response code" {response.status_code}')
					except Exception as e:
						print(
							f"Error uploading to Elsstic Search for: {network['name']} Error: {str(e)}")


"""
    Main Module of autosync Application
"""
import asyncio
from os import getenv
from merakitools import meraki_tasks
from merakitools import const, lib, model, utils
from merakitools.model import golden_nets, meraki_nets


def setup_app(cfg_file=None):
	const.appcfg = model.APPCONFIG(cfg_file)
	temp_sdk = lib.MerakiApi()
	const.meraki_sdk = temp_sdk.api


def run(cfg_file=None,task='sync',device_task=None,network_name=None):
	"""
    Main Module Start of Application
    Args:
    Returns:

    """
	setup_app(cfg_file)
	if task == 'sync':
		asyncio.run(meraki_tasks.sync_task(network_name=network_name))
		return "Sync Complete"
	elif task == 'device_config':
		asyncio.run(meraki_tasks.device_config(device_task))
	
	

if __name__ == '__main__':
	#config_file = '~/apps/testSync/ciscolab-config.json'
	config_file=None
	network_name = "29Q892"
	if config_file is None:
		config_file = getenv("MERAKI_TOOLS_CONFIG", None)
	#run(config_file,"sync",network_name="29Q892")
	run(config_file,"sync",network_name="29Q892,09X055,07X018,15B029,15B094,02M041")
	#run(config_file, "sync")
	print('Done')



from flask import Flask,abort, current_app, request, url_for,jsonify,make_response
from werkzeug.exceptions import HTTPException, InternalServerError
from flask_restful import Resource, Api
from datetime import datetime
from functools import wraps
import threading
import time
import uuid
from merakitools.main import run
from os import getenv


tasks = {}

app = Flask(__name__)
api = Api(app)


@app.before_first_request
def before_first_request():
    """Start a background thread that cleans up old tasks."""
    def clean_old_tasks():
        """
        This function cleans up old tasks from our in-memory data structure.
        """
        global tasks
        while True:
            # Only keep tasks that are running or that finished less than 5
            # minutes ago.
            five_min_ago = datetime.timestamp(datetime.utcnow()) - 5 * 60
            tasks = {task_id: task for task_id, task in tasks.items()
                     if 'completion_timestamp' not in task or task['completion_timestamp'] > five_min_ago}
            time.sleep(60)

    if not current_app.config['TESTING']:
        thread = threading.Thread(target=clean_old_tasks)
        thread.start()


def async_api(wrapped_function):
    @wraps(wrapped_function)
    def new_function(*args, **kwargs):
        def task_call(flask_app, environ):
            # Create a request context similar to that of the original request
            # so that the task can have access to flask.g, flask.request, etc.
            with flask_app.request_context(environ):
                try:
                    tasks[task_id]['return_value'] = wrapped_function(*args, **kwargs)
                except HTTPException as e:
                    tasks[task_id]['return_value'] = current_app.handle_http_exception(e)
                except Exception as e:
                    # The function raised an exception, so we set a 500 error
                    tasks[task_id]['return_value'] = InternalServerError()
                    if current_app.debug:
                        # We want to find out if something happened so reraise
                        raise
                finally:
                    # We record the time of the response, to help in garbage
                    # collecting old tasks
                    tasks[task_id]['completion_timestamp'] = datetime.timestamp(datetime.utcnow())

                    # close the database session (if any)

        # Assign an id to the asynchronous task
        task_id = uuid.uuid4().hex

        # Record the task, and then launch it
        tasks[task_id] = {'task_thread': threading.Thread(
            target=task_call, args=(current_app._get_current_object(),
                               request.environ))}
        tasks[task_id]['task_thread'].start()

        # Return a 202 response, with a link that the client can use to
        # obtain task status
        print(url_for('gettaskstatus', task_id=task_id))
        #return jsonify()f"ccepted location {url_for('gettaskstatus', task_id=task_id)}", 202, {'Location': url_for('gettaskstatus', task_id=task_id)}
        return make_response(jsonify({"message": "Proccess Accepted",
                                      "status": 202,
                                      "status url": url_for('gettaskstatus', task_id=task_id)}),
                             202,
                             {'Location': url_for('gettaskstatus', task_id=task_id)})
    return new_function


class GetTaskStatus(Resource):
    def get(self, task_id):
        """
        Return status about an asynchronous task. If this request returns a 202
        status code, it means that task hasn't finished yet. Else, the response
        from the task is returned.
        """
        task = tasks.get(task_id)
        if task is None:
            abort(404)
        if 'return_value' not in task:
            #return "Sync Still Running", 202, {'Location': url_for('gettaskstatus', task_id=task_id)}
            return make_response(jsonify({"message" : "Sync Still Running",
                                          "status" : 202,
                                          "status url": url_for(
                                              'gettaskstatus',
                                              task_id=task_id)}),
                                 202,
                                 {'Location': url_for('gettaskstatus',
                                                      task_id=task_id)})
        return make_response(jsonify({"message" : "Sync Complete",
                                          "status" : 200}),
                                 202)


class CatchAll(Resource):
    @async_api
    def get(self, path=''):
        # perform some intensive processing
        print("starting processing task, path: '%s'" % path)
        time.sleep(10)
        print("completed processing task, path: '%s'" % path)
        return f'The answer is: {path}'

class Sync(Resource):
    @async_api
    def get(self, path=''):
        # perform some intensive processing
        config_file = getenv("MERAKI_TOOLS_CONFIG", None)
        if config_file is None:
            return "No Config Filed Defined"
        run(config_file,task='sync',network_name="29Q892,09X055,07X018,15B029,15B094,02M041")
        #run(config_file, task='sync')
        print("starting processing task, path: '%s'" % path)
        time.sleep(10)
        print("completed processing task, path: '%s'" % path)
        return f'The answer is: {path}'

api.add_resource(CatchAll, '/<path:path>', '/')
api.add_resource(GetTaskStatus, '/status/<task_id>')
api.add_resource(Sync,'/sync')


if __name__ == '__main__':
    app.run(debug=True)


"""
Proccess Orginization to find networks with in Org then sends each network
to the Network Proccessor
"""
import asyncio
import logging
import sys
import threading
import time
from datetime import datetime
from merakitools import const, lib, model
from .processor_helper import proccess_network


class Orgsyncprocessor(threading.Thread):
	"""
    Stages the sync function to run the processor in ASYNC or load cache
     memeory or loads cache into memeory if using
    cache
    """
	
	def __init__(self, org_id,network_name=None):
		"""
        Init of Org Sync Pocessor
        Args:
            org_id:ID or Org to be proccessed
        """
		threading.Thread.__init__(self)
		self.org_id = org_id
		self.change_log = None
		self.networks = []
		if network_name is None:
			self.single_sync = False
			self.network_name =None
		else:
			self.single_sync = True
			self.network_name = network_name
	
	def run(self):
		start_time = time.perf_counter()
		threading.currentThread().setName(
				model.meraki_nets[self.org_id].name)
		self.change_log = asyncio.run(
				lib.get_change_log_from_org(self.org_id))
		if const.appcfg.debug:
			print(f'\tOrgName: {model.meraki_nets[self.org_id].name} Sync '
			      f'Started at: {start_time} '
			      f'Thread PID:{threading.currentThread().native_id}')
		
		nets = asyncio.run(self._get_org_networks())
		model.meraki_nets[self.org_id].net_count = int(len(nets))
		if len(nets) == 0:
			print (
				f"{lib.bc.FAIL} Tags are case sentive. Please validate network tags in Orginaztion {model.meraki_nets[self.org_id].name} "
				f"Golden: {const.appcfg.tag_golden} Target: {const.appcfg.tag_target}")
			return False
		if const.appcfg.use_cache:
			lib.load_cache(self.org_id, model.meraki_nets[self.org_id],
			               'autosync')
		has_golden = False
		if self.single_sync:
			for net in nets:
				if (str(net["name"]) in self.network_name) or (const.appcfg.tag_golden in net["tags"]):
					self.networks.append(net['id'])
		else:
			
			for net in nets:
				self.networks.append(net['id'])
				if (const.appcfg.tag_golden in net['tags'] or net[
					'id'] == 'L_575334852396597314') and not model.golden_nets[
					const.appcfg.tag_golden].cached:
					model.golden_nets[const.appcfg.tag_golden].networks[
						const.appcfg.tag_golden] = model.MNET(net)
					has_golden = True
					
				elif (const.appcfg.tag_golden in net['tags'] or net[
					'id'] == 'L_575334852396597314') and model.golden_nets[
					const.appcfg.tag_golden].cached:
					has_golden = True
					
				if (const.appcfg.tag_target in net[
					'tags'] or const.appcfg.tag_override) and not \
						model.meraki_nets[self.org_id].cached:
					model.meraki_nets[self.org_id].networks.update(
							{net['id']: model.MNET(net)})
				elif const.appcfg.tag_target in net['tags'] and net[
					'id'] not in model.meraki_nets[
					self.org_id].networks.keys():
					model.meraki_nets[self.org_id].networks.update(
							{net['id']: model.MNET(net)})
					
				elif net['id'] in model.meraki_nets[
					self.org_id].networks.keys():
					model.meraki_nets[self.org_id].networks[net['id']].tags = \
						net['tags']
		remove_list = []
	#	for net in model.meraki_nets[self.org_id].networks:
	#		if net not in self.networks:
	#			remove_list.append(net)
	#	for net in remove_list:
	#		model.meraki_nets[self.org_id].networks.pop(net)
		
		model.meraki_nets[self.org_id].sync_nets = self.networks
		model.meraki_nets[self.org_id].net_count = int(len(self.networks))
		asyncio.run(self._async_run(has_golden))
		
		model.meraki_nets[self.org_id].syncruntime = \
			time.perf_counter() - start_time
		model.meraki_nets[self.org_id].lastsync = datetime.utcnow()
		asyncio.run(lib.update_change_log(self.org_id))
		##TODO Check task
		lib.store_cache(self.org_id, model.meraki_nets[self.org_id],
		                'device_config')
		
		print(
				f'{lib.bc.OKBLUE} OrgName: {model.meraki_nets[self.org_id].name} '
				f'{lib.bc.WARNING}Loaded Last Sync Time: {model.meraki_nets[self.org_id].lastsync}{lib.bc.ENDC}'
		)
	
	async def _async_run(self, has_golden):
		with lib.MerakiAsyncApi() as sdk:
			logger = logging.getLogger('meraki.aio')
			logger.setLevel(logging.WARNING)
			
			model.meraki_nets[self.org_id].change_log = self.change_log
			if has_golden:
				model.golden_nets[
					const.appcfg.tag_golden].change_log = self.change_log
			
			net_tasks = [
					proccess_network(self.org_id, net, sdk,
					                 model.meraki_nets[
						                 self.org_id].networks[net].tags)
					for net in self.networks
			]
			await asyncio.gather(*net_tasks)
	
	async def _get_org_networks(self):
		with lib.MerakiAsyncApi() as sdk:
			return  await sdk.organizations.getOrganizationNetworks(
					self.org_id,tags=[const.appcfg.tag_golden,const.appcfg.tag_target],tagsFilterType="withAnyTags")
